//------------------------------------------------------------------------------
// File: MyRTSPFilter.h
//
// Desc: DirectShow sample code - definitions for MyRTSPFilter renderer.
//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------------------------

#include "ICallbackBuffer.h"
#include "FramedSource.hh"

// Because of differences between Visual C++ and older Microsoft SDKs,
// you may have defined _DEBUG without defining DEBUG.  This logic
// ensures that both will be set if Visual C++ sets _DEBUG.

extern PBYTE pbGReadBuffer; // Global Read Buffer.
extern long lGActualDataLength; // Extern Actual Sample DataLength


class CRTSPInputPin;
class CRTSP;
class CRTSPFilter;

#define BYTES_PER_LINE 20
#define FIRST_HALF_LINE TEXT  ("   %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x")
#define SECOND_HALF_LINE TEXT (" %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x")

//CRTSPInputPin *m_pPin;          // A simple rendered input pin

// Main filter object

class CRTSPFilter : public CBaseFilter
{
    CRTSP * const m_pRTSP;

public:

    // Constructor
    CRTSPFilter(CRTSP *pRTSP,
                LPUNKNOWN pUnk,
                CCritSec *pLock,
                HRESULT *phr);

    // Pin enumeration
    CBasePin * GetPin(int n);
    int GetPinCount();

    // Open and close the file as necessary
    STDMETHODIMP Run(REFERENCE_TIME tStart);
    STDMETHODIMP Pause();
    STDMETHODIMP Stop();
};


//  Pin object

class CRTSPInputPin :  public FramedSource, public CRenderedInputPin
{
    CRTSP    * const m_pRTSP;           // Main renderer object
    CCritSec * const m_pReceiveLock;    // Sample critical section
    REFERENCE_TIME m_tLast;             // Last sample receive time
//	ICallbackBuffer *pCallbackBufferPtr; // CallBack pointer 

public:
	
	CRTSPInputPin(UsageEnvironment& env,
				  CRTSP *pRTSP,
                  LPUNKNOWN pUnk,
                  CBaseFilter *pFilter,
                  CCritSec *pLock,
                  CCritSec *pReceiveLock,
                  HRESULT *phr);

    virtual ~CRTSPInputPin();

	static CRTSPInputPin* createNew(UsageEnvironment& env,
										CRTSP *pRTSP,
										LPUNKNOWN pUnk,
										CBaseFilter *pFilter,
										CCritSec *pLock,
										CCritSec *pReceiveLock,
										HRESULT *phr);

	static int FrameIndex;
	static int NotReadyFrames;

private:
	  virtual void doGetNextFrame();
public:
  void deliverFrame();

//	public:
//  //static MPEG1or2LiveSource* createNew(UsageEnvironment& env,DeviceParameters params);
//  static MPEG1or2LiveSource* createNew(UsageEnvironment& env);  
//  static int FrameIndex;
//
//public:
//  //MPEG1or2LiveSource(UsageEnvironment& env, DeviceParameters params);
//  MPEG1or2LiveSource(UsageEnvironment& env);
//
//  // called only by createNew(), or by subclass constructors
//  virtual ~MPEG1or2LiveSource();
// // virtual void CallFunction(BYTE *);
//
//private:
////protected:
//  // redefined virtual functions:
//  virtual void doGetNextFrame();
//
////private:
//public:
//  void deliverFrame();
//
   

    // Do something with this media sample
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);
    STDMETHODIMP ReceiveCanBlock();

    // Write detailed information about this sample to a file
    HRESULT WriteStringInfo(IMediaSample *pSample);

    // Check if the pin can support this specific proposed type and format
    HRESULT CheckMediaType(const CMediaType *);

    // Break connection
    HRESULT BreakConnect();

    // Track NewSegment
    STDMETHODIMP NewSegment(REFERENCE_TIME tStart,
                            REFERENCE_TIME tStop,
                            double dRate);

	static int MediaSampleCounter;  // MediaSampleCounter
};


//  CRTSP object which has filter and pin members

class CRTSP : public CUnknown, public IFileSinkFilter
{
    friend class CRTSPFilter;
    friend class CRTSPInputPin;

    CRTSPFilter   *m_pFilter;       // Methods for filter interfaces
    CRTSPInputPin *m_pPin;          // A simple rendered input pin

    CCritSec m_Lock;                // Main renderer critical section
    CCritSec m_ReceiveLock;         // Sublock for received samples

    CPosPassThru *m_pPosition;      // Renderer position controls

    HANDLE   m_hFile;               // Handle to file for RTSPing
    LPOLESTR m_pFileName;           // The filename where we RTSP
    BOOL     m_fWriteError;

public:

    DECLARE_IUNKNOWN

    CRTSP(LPUNKNOWN pUnk, HRESULT *phr);
    ~CRTSP();

    static CUnknown * WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);

    // Write string, followed by CR/LF, to a file
    void WriteString(TCHAR *pString);

    // Write raw data stream to a file
    HRESULT Write(PBYTE pbData, LONG lDataLength);

    // Implements the IFileSinkFilter interface
    STDMETHODIMP SetFileName(LPCOLESTR pszFileName,const AM_MEDIA_TYPE *pmt);
    STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName,AM_MEDIA_TYPE *pmt);

private:

    // Overriden to say what interfaces we support where
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);

    // Open and write to the file
    HRESULT OpenFile();
    HRESULT CloseFile();
    HRESULT HandleWriteFailure();
};

